@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,8 +32,8 @@
  *  Software Distribution Coordinator  or  sdc@inet.no
  *  Inferno Nettverk A/S
  *  Oslo Research Park
- *  Gaustadalen 21
- *  N-0349 Oslo
+ *  Gaustadalllllen 21
+ *  NO-0349 Oslo
  *  Norway
  *
  * any improvements or extensions that they make and grant Inferno Nettverk A/S
@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: Rbind.c,v 1.103 2000/05/31 12:14:51 karls Exp $";
+"$Id: Rbind.c,v 1.106 2001/02/06 15:58:46 michaels Exp $";
 
 int
 Rbind(s, name, namelen)
@@ -99,7 +99,7 @@ Rbind(s, name, namelen)
 		}
 	}
 
-	/* hack for performance (testing). */
+	/* hack for performance testing. */
 	if (getenv("SOCKS_BINDLOCALONLY") != NULL)
 		return rc;
 
@@ -153,6 +153,8 @@ Rbind(s, name, namelen)
 	if (socks_requestpolish(&packet.req, NULL, NULL) == NULL)
 		return 0;	/* socket bound, assume ok. */
 
+	packet.version = packet.req.version;
+
 	switch (packet.req.version) {
 		case SOCKS_V4:
 		case SOCKS_V5: {
